<?php
/** 
 * @author 42functions | Ferdy Perdaan
 * @version 1.0
 * 
 * GUIFactory used to easy create form elements.
 * 
 * The following options can be used on every element unless otherwise is indicated:
 * - string id The identification of the element.
 * - string name The name of the element.
 * - string label The label attached to the element.
 * - string class Additional classes to append to the element.
 * - string style The style of the element.
 * - string|int|bool value The default value of the element.
 */
 
if(defined('XLII_GUIFACTORY_INIT'))
    return;

define('XLII_GUIFACTORY_INIT', true);

class XLII_GUIFactory
{
    const SOURCE_NONE = 'none';
    const SOURCE_BOTH = 'both';
    const SOURCE_POST = 'post';
    const SOURCE_GET = 'get';
    
    private static $_source = self::SOURCE_POST;
    private static $_subcounter = array();
    
    /** INIT RESOURCES ----------------------------------------------------------------------------------------------------------------------------------------
     * Initialize all the resources 
     */
    public static function initResources()
    {
        // Wordpress core is depricated, we need a new one
        wp_register_script('xlii_guifactory', substr(dirname(__FILE__), strlen(ABSPATH) - 1).'/xlii_guifactory.js', array('jquery'));
        wp_register_script('xlii_ui_core', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.14/jquery-ui.min.js', array(), 1.0, true);

        wp_register_style('xlii_guifactory', substr(dirname(__FILE__), strlen(ABSPATH) - 1).'/xlii_guifactory.css', array('xlii_ui_style'));
        wp_register_style('xlii_ui_style_basic', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.14/themes/base/jquery-ui.css', 1.0, true);
        wp_register_style('xlii_ui_style', 
            'http://jqueryui.com/themeroller/css/parseTheme.css.php?ffDefault=Trebuchet+MS,+Tahoma,+Verdana,+Arial,+sans-serif&fwDefault=bold&fsDefault=1.1em&cornerRadius=4px&bgColorHeader=f6a828&bgTextureHeader=12_gloss_wave.png&bgImgOpacityHeader=35&borderColorHeader=e78f08&fcHeader=ffffff&iconColorHeader=ffffff&bgColorContent=eeeeee&bgTextureContent=03_highlight_soft.png&bgImgOpacityContent=100&borderColorContent=dddddd&fcContent=333333&iconColorContent=222222&bgColorDefault=f6f6f6&bgTextureDefault=02_glass.png&bgImgOpacityDefault=100&borderColorDefault=cccccc&fcDefault=1c94c4&iconColorDefault=ef8c08&bgColorHover=fdf5ce&bgTextureHover=02_glass.png&bgImgOpacityHover=100&borderColorHover=fbcb09&fcHover=c77405&iconColorHover=ef8c08&bgColorActive=ffffff&bgTextureActive=02_glass.png&bgImgOpacityActive=65&borderColorActive=fbd850&fcActive=eb8f00&iconColorActive=ef8c08&bgColorHighlight=ffe45c&bgTextureHighlight=03_highlight_soft.png&bgImgOpacityHighlight=75&borderColorHighlight=fed22f&fcHighlight=363636&iconColorHighlight=228ef1&bgColorError=b81900&bgTextureError=08_diagonals_thick.png&bgImgOpacityError=18&borderColorError=cd0a0a&fcError=ffffff&iconColorError=ffd27a&bgColorOverlay=666666&bgTextureOverlay=08_diagonals_thick.png&bgImgOpacityOverlay=20&opacityOverlay=50&bgColorShadow=000000&bgTextureShadow=01_flat.png&bgImgOpacityShadow=10&opacityShadow=20&thicknessShadow=5px&offsetTopShadow=-5px&offsetLeftShadow=-5px&cornerRadiusShadow=5px', 
            array('xlii_ui_style_basic'),
            1.0,
            true
        );
    }
    
    /** _ATTRIBUTES -------------------------------------------------------------------------------------------------------------------------------------------
     * Return the attributes stored within the options.
     *
     * @param   array &$options The options to fetch the shared data from.
     * @param   array $include = null An array containing the allowed fields.
     * @return  string
     */
    protected static function _attributes(array &$options, array $include = null)
    {
        if(isset($options['appendClass']))
        {
            $options['class'] = (isset($options['class']) ? $options['class'].' ' : '').$options['appendClass'];
            unset($options['appendClass']);
        }
            
        if(isset($options['name']) && !isset($options['id']))
            $options['id'] = $options['name'];
            
        $options['id'] = str_replace('[]', '', $options['id']);
        $options['id'] = str_replace(array('][', ']', '['), '_', $options['id']);
            
        $attr = array();
        foreach(array('name', 'id', 'class', 'style', 'onchange', 'onclick') as $type)
        {
            if(isset($options[$type]) && (!$include || in_array($type, $include)))
                $attr[] = $type.' = "'.$options[$type].'"';
        }
        
        return count($attr) ? ' '.implode(' ', $attr) : '';
    }
    
    /** CALENDAR ----------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a default calendar selector.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function calendar($options)
    {
        wp_enqueue_script('xlii_ui_core');
        wp_enqueue_style('xlii_ui_style');
        
        $options['class'] = (isset($options['class']) ? $options['class'].' ' : '').' xlii-calendar';
        
        return self::text($options);
    }
    
    /** CHECKBOX ----------------------------------------------------------------------------------------------------------------------------------------------
     * Return a checkbox.
     *
     * @param   array $options An array containing element specific options.
     *          Allowed options
     *          - bool value Indicate wether the checkbox is checked or not.
     * @return  string
     */
    public static function checkbox(array $options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-checkbox';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : false);
        
        return '<input type = "checkbox"'.self::_attributes($options).($options['value'] ? ' checked = "checked"' : '').' />'.self::label($options);
    }

    /** ELEMENT -----------------------------------------------------------------------------------------------------------------------------------------------
     * Global render method, specify a type for fast dynamic rendering.
     *
     * @param   enum $type The type of element to render.
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function element($type, array $options)
    {
        if(method_exists(__CLASS__, $type) && $type[0] != '_' && $type != 'element')
            return self::$type($options);
        else
            return '';
    }
    
    /** HIDDEN ------------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a default hidden element.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function hidden($options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-hidden';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        
        return '<input type = "hidden"'.self::_attributes($options).($options['value'] ? ' value = "'.self::_protect($options['value']).'"' : '').' />';
    }
    
    /** LABEL -------------------------------------------------------------------------------------------------------------------------------------------------
     * Return a label for an element.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function label(array $options)
    {
        if(isset($options['label']))
            return '<label for = "'.(isset($options['id']) ? $options['id'] : $options['name']).'"'.self::_attributes($options, array('class', 'style')).'>'.$options['label'].'</label>';
        else
            return '';
    }
    
    /** _PROTECT ----------------------------------------------------------------------------------------------------------------------------------------------
     * Parse user given data to make sure its safe for output.
     *
     * @param   string $text The text to parse.
     * @return  string
     */
    protected function _protect($text)
    {
        return htmlentities($text);
    }
    
    /** RADIO -------------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a radiobutton.
     *
     * @param   array $options An array containing element specific options.
     *          Allowed options
     *          - array options An array containing values => labels used to create entire radio groups at once. 
     * @return  string
     */
    public static function radio(array $options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-radio';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        
        if(isset($options['options']))
        {   
            $html = '<div class = "radiogroup">';
            $i = 0;
            foreach($options['options'] as $value => $label)
            {
                $options['id'] = $options['name'].'_'.(++$i);
                $options['label'] = $label;
                $html .= '<div class = "entity"><input type = "radio"'.self::_attributes($options).' value = "'.$value.'"'.($options['value'] == $value ? ' checked = "checked"' : '').' />'.self::label($options).'</div>';
            }
                
                
            return $html.'</div>';
        }
        else
        {
            return '<input type = "radio"'.self::_attributes($options).' value = "'.$options['value'].'"'.(self::_value($options['name'], $options['value']) == $options['value'] ? ' checked = "checked"' : '').' />'.self::label($options);
        }
    }   
    
    /** SELECT ------------------------------------------------------------------------------------------------------------------------------------------------
     * Return a selectbox.
     *
     * @param   array $options An array containing element specific options.
     *          Allowed options
     *          - bool multi = false Indicate wether multiple values are allowed to be selected.
     *          - array options The options within the selectbox value => label.
     * @return  string
     */
    public static function select(array $options)
    {   
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-select';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        $options['value'] = !is_array($options['value']) ? array($options['value']) : $options['value'];
     
        
        if(isset($options['multi']) && $options['multi'])
        {
            $options['appendClass'] .= ' multi';
            if(substr($options['name'], -2) != '[]')
            {
                if(!isset($options['id']))
                    $options['id'] = $options['name'];
                $options['name'] .= '[]';
            }
        }
        
        $html = self::label($options).'<select'.self::_attributes($options).(isset($options['multi']) && $options['multi'] ? 'multiple="multiple"' : '').'>';
        foreach($options['options'] as $value => $label)
            $html .= '<option value = "'.$value.'"'.(in_array($value, $options['value']) ? 'selected = "selected"' : '').'>'.$label.'</option>';
        
        return $html.'</select>';
    }
    
    /** SELECT ------------------------------------------------------------------------------------------------------------------------------------------------
     * Return a selectbox.
     *
     * @param   array $options An array containing element specific options.
     *          Allowed options
     *          - bool multi = true Indicate wether multiple values are allowed to be selected.
     *          - bool linked = false Only allow to select linked media.
     *          - bool popup = true Indicate wether the files are selected trough a popupbox
     *          - array mime_type The allowed mime types that may be selected.
     * @return  string
     */
    public static function selectMedia(array $options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        $options['value'] = !is_array($options['value']) ? array($options['value']) : $options['value'];
       
        $media = get_posts(array(
            'post_type' => 'attachment', 
            'numberposts' => -1, 
            'order' => 'ASC', 
            'orderby' => 'title', 
            'post_mime_type' => isset($options['mime_type']) ? $options['mime_type'] : null,
            'post_parent' => isset($options['linked']) && $options['linked'] ? get_the_ID() : null
        ));
        
        $type = !isset($options['multi']) || $options['multi'] ? 'checkbox' : 'radio';
        $tabs = array(
            'Active' => '',
        );
        
        // Collect tabs
        foreach($media as $post)
        {
            $url = wp_get_attachment_image_src($post->ID, array(50, 50));
            $row = '<li class = "tab '.sanitize_title($post->post_mime_type).(in_array($post->ID, $options['value']) ? ' active' : '').'"><input type = "'.$type.'" id = "'.$options['name'].'_'.$post->ID.'" name = "'.$options['name'].'[]" value = "'.$post->ID.'" class = "xlii-'.$type.' xlii-input" '.(in_array($post->ID, $options['value']) ? 'checked="checked"' : '').' /><span class = "image"><img src = "'.$url[0].'" width = "'.$url[1].'" height = "'.$url[2].'" /></span><span class = "title">'.get_the_title($post->ID).'</span><span class = "full">'.wp_get_attachment_url($post->ID).'</span></li>';
        
            if(!$tabs[$post->post_mime_type])
                $tabs[$post->post_mime_type] = $row;
            else
                $tabs[$post->post_mime_type] .= $row;
        }
        
        // Gather content
        $navigation = $container = '';
        foreach($tabs as $title => $content)
        {
            $navigation .= '<li class = "nav" rel = "'.sanitize_title($title).'">'.$title.'</li>';
            $container  .= $content;
        }
        
        if($popup = !isset($options['popup']) || $options['popup'])
        {
            $reload = '';
            foreach($options['value'] as $post)
            {
                $url = wp_get_attachment_image_src($post, array(50, 50));
                $reload .= '<li class = "tab '.sanitize_title($post->post_mime_type).' active" rel = "'.$post.'"><span class = "image"><img src = "'.$url[0].'" width = "'.$url[1].'" height = "'.$url[2].'" /></span><span class = "title">'.get_the_title($post).'</span><span class = "full">'.wp_get_attachment_url($post).'</span></li>';
            }
        }
        
        echo '<div class = "xlii-mediaselector'.($popup ? ' popup' : '').'">
                <div class = "element xlii-popup" id = "xlii-mediaselector-'.md5($options['id']).'">
                    <ul class = "navigation">'.$navigation.'</ul>
                    <ul class = "content">'.$container.'</ul>
                    <div class = "controlls">
                        <span class = "prev item">'.__('Previous').'</span>
                        <span class = "next item">'.__('Next').'</span>
                        <input type = "hidden" class = "page" value = "0" />
                    </div>
                    <div class = "searchbox"><input type = "text" class = "xlii-search" /></div>
                </div>
                '.($popup ? '<div class = "frontend"><ul class = "active">'.$reload.'</ul><a class = "change xlii-showPopup" href = "xlii-mediaselector-'.md5($options['id']).'">'.__('Select media files').'</a></div>' : '').'
              </div>';
              
        if($popup)
            self::supportPopup(true);
    }
    
    /** SET SOURCE --------------------------------------------------------------------------------------------------------------------------------------------
     * Set the source used for fetching the elements values.
     *
     * @param   enum $type The source type to fetch values from.
     */
    public static function setSource($type)
    {
        self::$_source = $type;
    }
    
    /** SUBMIT ------------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a default submission element.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function submit($options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-submit button';
        $options['value'] = isset($options['value']) ? $options['value'] : 'Submit';
        $options['label'] = isset($options['label']) ? $options['label'] : $options['value'];
        
        return '<input type = "submit"'.self::_attributes($options).' value = "'.self::_protect($options['label']).'" />';
    }
    
    /** SUPPORT POPUP ----------------------------------------------------------------------------------------------------------------------------------------
     * Support popup functionality for the guifactory.
     */
    public static function supportPopup($add = false)
    {
        static $added;
        if($add && !$added)
        {
            $added = true;
            add_action('admin_footer', __CLASS__.'::'.__FUNCTION__);
        }
        else
        {
            echo '<div id = "xlii-popupOverlay"></div>';
        }   
    }
    
    /** TEXT --------------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a default text element.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function text($options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-text';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        
        return self::label($options).'<input type = "text"'.self::_attributes($options).($options['value'] ? ' value = "'.self::_protect($options['value']).'"' : '').' />';
    }
    
    /** TEXTAREA -----------------------------------------------------------------------------------------------------------------------------------------------
     * Returns a default textarea element.
     *
     * @param   array $options An array containing element specific options.
     * @return  string
     */
    public static function textarea($options)
    {
        if(!isset($options['name']))    $options['name'] = $options['id'];
        
        $options['appendClass'] = 'xlii-textarea';
        $options['value'] = self::_value($options['name'], isset($options['value']) ? $options['value'] : '');
        
        return self::label($options).'<br /><textarea'.self::_attributes($options).'>'.stripslashes($options['value'] ? $options['value'] : '').'</textarea>';
    }
    
    /** _VALUE ------------------------------------------------------------------------------------------------------------------------------------------------
     * Returns the value of an element.
     *
     * @param   string $name The name of the element.
     * @param   void $default The default value.
     * @return  void|null
     */
    protected static function _value($name, $default = null)
    {
        if(strpos($name, '[') !== false && self::$_source == self::SOURCE_NONE)
        {
            $name   = explode('[', str_replace(']', '', $name));
            $chain  = '';
            if(self::$_source == self::SOURCE_BOTH)
                $source = array_merge($_POST, $_GET);
            else if(self::$_source == self::SOURCE_POST)
                $source = $_POST;
            else
                $ousrce = $_GET;
            
            foreach($name as &$key)
            {
                // Make sure we have a valid key
                if(!$key)
                {
                    if(!isset(self::$_subcounter[$chain]))
                        $key = self::$_subcounter[$chain] = 0;
                    else
                        $key = self::$_subcounter[$chain];
                        
                    self::$_subcounter[$chain]++;
                }
                
                if(!isset($source[$key]))
                    return $default;
                
                $source = $source[$key];
            }    
            return $source;
        }
        else
        {
            if(isset($_POST[$name]) && (self::$_source == self::SOURCE_BOTH || self::$_source == self::SOURCE_POST))
                return $_POST[$name];
            else if(isset($_GET[$name]) && (self::$_source == self::SOURCE_BOTH || self::$_source == self::SOURCE_GET))
                return $_GET[$name];
        }
        return $default;    
    }
}

XLII_GUIFactory::initResources();
